/*
 *  Copyright (C) Samuel Thibault <samuel.thibault@fnac.net>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 * analyse.h
 *
 * a ppp analyser for Unix
 *
 * (c) Copyright 2000-2001 Samuel Thibault
 */

#include <stdlib.h>

#define maxline		255

#ifndef RUNTIME
#define exit(n)	while(1);	// so as to be able to do Alt-SysReq-B
#endif

/*
 * u16 represents an unsigned 16-bit number.  Adjust the typedef for
 * your hardware.
 */
typedef unsigned short u16;
 
/*
 * for finding unsupported protocols names by their codes
 */
struct protos_s {
	short code;
	char *info;
};

/*
 * what will we have to show ?
 */
extern struct show_s {
	short xonxoff;
	short asyncmap;
	short cp;
	short auth;
	short ip;
	short icmp;
	short udp;
	short tcp;
	short domain;
	short others;
} show;

/*
 * adds s to the future loggued string
 */
void loggue(unsigned char *s);

/*
 * initializes automates & log
 */
void trame_begin(void);

/*
 * prints the log string prepared thanks to loggue
 */
void flush_log(void);

/*
 * *the* PPP analyser
 */
void analyse(unsigned char c); 

/*
 * analyses cleaned chars
 */
void analysechar(unsigned char c); 

/*
 * upper layers
 */
void analysecp(unsigned char c);
void analysepap(unsigned char c);
void analysechap(unsigned char c);
void analyseip(unsigned char c);
void analyseicmp(unsigned char c);
void analysetcp(unsigned char c);
void analyseudp(unsigned char c);
void analysedomain(unsigned char c);

/*
 * finds the string corresponding to code, or "unknown: 0x2c3" if not found
 */
char *findproto(struct protos_s *protos, short unsigned code);

/*
 * finds the string corresponding to code, or "80" if not found
 */
char *findservice(unsigned int code);

/*
 * ppp stuff for reliability
 */
u16 pppfcs(u16 fcs, unsigned char c);
