/*
 *  Copyright (C) Samuel Thibault <samuel.thibault@fnac.net>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 * auth.c
 *
 * a ppp analyser for Unix
 *
 * (c) Copyright 2000-2001 Samuel Thibault
 */

#include <stdio.h>
#include "analyse.h"
#include "codes.h"

#define AUTHloggue(s)	if (show.auth) loggue(s);

short stateauth;
short statepap;
short statechap;
short authcontent;
int authlength;
int paplength;
int chaplength;

void analysepap(unsigned char c) {
	static char s[64];

	switch (stateauth) {
	case SAUTH_CODE:
		AUTHloggue(" PAP");
		switch (c) {
		case 1:
			AUTHloggue(" Req");
			break;
		case 2:
			AUTHloggue(" Ack");
			break;
		case 3:
		       	AUTHloggue(" Nak");
			break;
		}
		authcontent=SAUTH_LENTXT;
		statepap=SPAP_LEN;
		stateauth=SAUTH_IDENTIFIER;
		break;
	case SAUTH_IDENTIFIER:
		sprintf(s," id=0x%x",c);
		AUTHloggue(s)
		stateauth=SAUTH_LENGTHHI;
		break;
	case SAUTH_LENGTHHI:
		authlength=c;
		stateauth=SAUTH_LENGTHLO;
		break;
	case SAUTH_LENGTHLO:
		authlength=(authlength << 8) + c - 4;
		stateauth=authcontent;
		break;
	case SAUTH_LENTXT:
		if (!--authlength) stateauth=discard;
		switch (statepap) {
		case SPAP_LEN:
			paplength=c;
			statepap=SPAP_TXT;
			AUTHloggue(" ");
			break;
		case SPAP_TXT:
			sprintf(s,"%c",c);
			AUTHloggue(s);
			if (!--paplength && authlength) {
				AUTHloggue(":");
				statepap=SPAP_LEN;
			}
			break;
		}
		break;
	case discard:
		break;
	}
}

void analysechap(unsigned char c) {
	static char s[64];

	switch (stateauth) {
	case SAUTH_CODE:
		AUTHloggue("CHAP");
		switch (c) {
		case 1:
		case 2:
			AUTHloggue(c==1?" Challenge":" Response");
			authcontent=SAUTH_CHALRESP;
			statechap=SCHAP_VSIZE;
			stateauth=SAUTH_IDENTIFIER;
			break;
		case 3:
		case 4:
		       	AUTHloggue(c==3?" Success":" Failure");
			authcontent=SAUTH_TXT;
			stateauth=SAUTH_IDENTIFIER;
			break;
		}
		break;
	case SAUTH_IDENTIFIER:
		sprintf(s," id=0x%x",c);
		AUTHloggue(s)
		stateauth=SAUTH_LENGTHHI;
		break;
	case SAUTH_LENGTHHI:
		authlength=c;
		stateauth=SAUTH_LENGTHLO;
		break;
	case SAUTH_LENGTHLO:
		authlength=(authlength << 8) + c-4;
		stateauth=authcontent;
		if (stateauth==SAUTH_TXT && authlength)
			AUTHloggue(" \"");
		break;
	case SAUTH_CHALRESP:
		if (!authlength--) stateauth=discard;
		switch (statechap) {
		case SCHAP_VSIZE:
			chaplength=c;
			statechap=SCHAP_VALUE;
			break;
		case SCHAP_VALUE:
			sprintf(s,"%x",c);
			AUTHloggue(s);
			if (authlength) {
				statechap=SCHAP_NAME;
				AUTHloggue(" from ");
			}
			break;
		case SCHAP_NAME:
			sprintf(s,"%c",c);
			AUTHloggue(s);
			break;
		}
		break;
	case SAUTH_TXT:
		if (authlength) {
			sprintf(s,"%c",c);
			AUTHloggue(s);
			if (!authlength--) {
				AUTHloggue("\"");
				stateauth=discard;
			}
		}
		break;
	case discard:
		break;
	}
}
