/*
 *  Copyright (C) Samuel Thibault <samuel.thibault@fnac.net>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 * cp.c
 *
 * a ppp analyser for Unix
 *
 * (c) Copyright 2000-2001 Samuel Thibault
 */

#include <stdio.h>
#include "analyse.h"
#include "codes.h"

#define CPloggue(s)	if (show.cp) loggue(s);

extern short state;
short statecp;
short stateoptioncp;

int cplength;			// length of the cp frame
int optionlength;		// length of the option
short curcp;			// current cp code
short cpcontent;		// its content
short curoption;		// current option code
short cpoptioncontent;		// its content
unsigned long int integer;
unsigned short cpadrip[4];

struct defcp_s {
	short code;
	char *info;
	short content;
} defcp [] = {
	{ 1,	"ConfReq",		SCP_OPTIONS },
	{ 2,	"ConfAck",		SCP_OPTIONS },
	{ 3,	"ConfNak",		SCP_OPTIONS },
	{ 4,	"ConfRej",		SCP_OPTIONS },
	{ 5,	"TermReq",		SCP_TXT },
	{ 6,	"TermAck",		SCP_TXT },
	{ 7,	"CodeRej",		discard },
	{ 8,	"Proto-Rej",		discard },
	{ 9,	"EchoReq",		SCP_MAGIC },
	{ 10,	"EchoRep",		SCP_MAGIC },
	{ 11,	"Discard",		discard },
	{ 12,	"Identification",	discard },
	{ 13,	"Time-Remaining",	discard },
	{ -1,	NULL,			0 }
};

struct defcpoptions_s {
	short code;
	char *info;
	short content;
} defoptionscp [2][16] = {
	{
		{ 1,	"mru",		COCP_NUMINT },
		{ 2,	"asyncmap",	COCP_BININT },
		{ 3,	"auth",		COCP_AUTHOPT },
		{ 4,	"quality",	COCP_QUALOPT },
		{ 5,	"magic",	COCP_BININT },
		{ 6,	"RESERVED",	discard },
		{ 7,	"pcomp",	discard },
		{ 8,	"accomp",	discard },
		{ 9,	"FCSalt",	COCP_FCSALT },
		{ 10,	"SDP",		COCP_SDP },
		{ 13,	"CBCP",		discard },
		{ 15,	"CF",		discard },
		{ 17,	"MRRU",		discard },
		{ 18,	"SSNHF",	discard },
		{ 19,	"EPDISC",	discard },
		{ -1,	NULL,		discard }
	},
	{
		{ 1,	"addrs",	COCP_ADRIP },
		{ 2,	"compr",	COCP_COMPR },
		{ 3,	"addr",		COCP_ADRIP },
		{ 129,	"dns1",		COCP_ADRIP },
		{ 130,	"wins1",	COCP_ADRIP },
		{ 131,	"dns2",		COCP_ADRIP },
		{ 132,	"wins2",	COCP_ADRIP },
		{ -1,	NULL,		discard }
	}
};

void analysecp(unsigned char c) {
	static char s[64];
	struct defcp_s *cur;
	struct defcpoptions_s *curopt;
	static short echap;

	switch (statecp) {
	case SCP_CODE:		// first a 8-bits code
		integer=0;
		for (cur=defcp;(cur->code!=-1) && (c != cur->code);cur++);
		if (cur->code != -1) {
			CPloggue(" ");
			CPloggue(cur->info);
			cpcontent=cur->content;
			statecp=SCP_IDENTIFIER;
		} else {
			sprintf(s," unknown: %#x",c);
			CPloggue(s);
			state=discard;
		}
		break;
	case SCP_IDENTIFIER:	// then a unique indentifier
		sprintf(s," id=0x%x",c);
		CPloggue(s);
		statecp=SCP_LENGTHHI;
		break;
	case SCP_LENGTHHI:	// a 16-bits length
		cplength=c;
		statecp=SCP_LENGTHLO;
		break;
	case SCP_LENGTHLO:
		cplength=(cplength << 8)+c-4;
		statecp=cpcontent;	// cpcontent was set 19 lines upper
		stateoptioncp=SOCP_TYPE;
		if (statecp==SCP_TXT && cplength)
			CPloggue(" \"");
		break;
	case SCP_OPTIONS:	// now options !
		if (!(--cplength)) statecp=discard;
		switch (stateoptioncp) {
		case SOCP_TYPE:		// its type first
			CPloggue(" <");
			for (curopt=defoptionscp[curcp];(curopt->code!=-1) && (c != curopt->code);curopt++);
			if (curopt->code != -1) {
				CPloggue(curopt->info);
				cpoptioncontent=curopt->content;
			} else {
				sprintf(s,"%#x>",c);
				CPloggue(s);
				cpoptioncontent=discard;
			}
			stateoptioncp=SOCP_LENGTH;
			break;
		case SOCP_LENGTH:	// then its 8-bits size
			optionlength=c-2;
			if (!optionlength) {
				CPloggue(">");
				stateoptioncp=SOCP_TYPE;	// next option !
			} else {
				stateoptioncp=SOCP_DATA;
				integer=0;
				echap=0xFF;
			}
			break;
		case SOCP_DATA:		// and its data
			optionlength--;
			switch (cpoptioncontent) {
			case COCP_NUMINT:
			case COCP_BININT:
				integer=(integer << 8) + c;
				break;
			case COCP_ADRIP:
				if (optionlength==3) CPloggue(" ");
				sprintf(s,"%d%s", c, optionlength ? "." : "");
				CPloggue(s);
				break;
			case COCP_AUTHOPT:
				switch (echap) {
				case 0xFF:	// was set 0xFF first 15 lines upper
					echap=c;
					break;
				case 0xFE:	// for chap algorithm
					switch (c) {
					case 0x05:
						CPloggue(" MD5");
						break;
					default:
						sprintf(s," 0x%x",c);
						CPloggue(s);
						break;
					}
					break;
				case 0xC0:
					switch (c) {
					case 0x23:
						CPloggue(" pap");
						break;
					default:
						sprintf(s," 0x%x",(echap << 8) + c);
						CPloggue(s);
						break;
					}
					break;
				case 0xC2:
					switch (c) {
					case 0x23:
						CPloggue(" chap");
						echap=0xFE;
						break;
					default:
						sprintf(s," 0x%x",(echap << 8) + c);
						CPloggue(s);
						break;
					}
					break;
				default:
					sprintf(s," 0x%x",c);
					CPloggue(s);
					break;
				}
				break;
			case COCP_QUALOPT:
				switch (echap) {
				case 0xFF:	// was set 0xFF 45 lines upper
					echap=c;
					break;
				case 0xC0:
					switch (c) {
					case 0x25:
						CPloggue(" LQR");
						break;
					default:
						printf(s," 0x%x",(echap << 8) + c);
						CPloggue(s);
						break;
					}
					break;
				default:
					sprintf(s," 0x%x",c);
					CPloggue(s);
					break;
				}
				break;
			case COCP_COMPR:
				switch (echap) {
				case 0xFF:	// was set 0xFF 65 lines upper
					echap=c;
					break;
				case 0x00:
	 				switch (c) {
					case 0x2d:
						CPloggue(" VJ");
						break;
					case 0x37:
						CPloggue(" old-VJ");
						break;
					default:
						sprintf(s," %02x",c);
						CPloggue(s);
						break;
					}
					break;
				default:
					sprintf(s," %02x:%02x", echap, c);
					CPloggue(s);
					break;
				}
				break;
			case COCP_FCSALT:
				if (c & 0x01) CPloggue (" null");
				if (c & 0x02) CPloggue (" 16");
				if (c & 0x04) CPloggue (" 32");
				break;
			case COCP_SDP:
				sprintf(s," %d",c);
				CPloggue(s);
				break;
			case discard:
				sprintf(s," 0x%x",c);
				CPloggue(s);
				break;
			}
			if (!optionlength) {	// what to display for finishing
				switch (cpoptioncontent) {
				case COCP_NUMINT:
					sprintf(s," %ld",integer);
					CPloggue(s);
					break;
				case COCP_BININT:
					sprintf(s," %#lx",integer);
					CPloggue(s);
					break;
				}
				CPloggue(">");
				stateoptioncp=SOCP_TYPE;	// next option
			}
			break;
		}
		break;
	case SCP_TXT:
		if (cplength) {
			sprintf(s,"%c",c);
			CPloggue(s);
			if (!cplength--) {
				CPloggue("\"");
				statecp=discard;
			}
		}
		break;
	case SCP_MAGIC:
		if (cplength) {
			integer=(integer << 8) + c;
			if (!cplength--) {
				sprintf(s," magic=%#lx",integer);
				CPloggue(s);
				statecp=discard;
			}
		}
		break;
	case discard:
		break;
	}
}
