/*
 *  Copyright (C) Samuel Thibault <samuel.thibault@fnac.net>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 * domain.c
 *
 * a ppp analyser for Unix
 *
 * (c) Copyright 2000-2001 Samuel Thibault
 */

#include <stdio.h>
#include "analyse.h"
#include "codes.h"

#define DOMAINloggue(s)	if (show.domain) loggue(s);

#define BEGIN	0
#define NAME	1
#define TYPEANDCO	2

extern short applipointeur;
short domainpointeur;

void analysedomain(unsigned char c) {
	static char s[64];
	static unsigned short w16;
	static unsigned long w32;
	static signed short current, questions, answers, authority, additionnal;
	static char state;
	static signed short namelen;
	static unsigned short type;
	static unsigned short size;
	switch(applipointeur) {
	case 0:
		DOMAINloggue(" domain");
	case 4:
	case 6:
	case 8:
	case 10:
		w16=c;
		break;
	/* ID */
	case 1:
		w16=(w16<<8)|c;
		sprintf(s,"(%u)",w16);
		DOMAINloggue(s);
		break;
	/* flags */
	case 2:
		switch ((c>>3)&15) {
		case 1: DOMAINloggue(" i"); break;
		case 2: DOMAINloggue(" status"); break;
		}
		if (!(c&128)) DOMAINloggue("req");
		if (c&4) DOMAINloggue(" auth");
		if (c&2) DOMAINloggue(" trunc");
		if (c&1) DOMAINloggue(" rec");
		break;
	/* flags again */
	case 3:
		if (c&7) DOMAINloggue(" recav");
		switch (c&15) {
		case 1: DOMAINloggue(" formaterr"); break;
		case 2: DOMAINloggue(" servfail"); break;
		case 3: DOMAINloggue(" nameerror"); break;
		case 4: DOMAINloggue(" unimpl"); break;
		case 5: DOMAINloggue(" refused"); break;
		}
		break;
	/* nb questions */
	case 5:
		questions=(w16<<8)|c;
		sprintf(s,"(%u",questions);
		DOMAINloggue(s);
		break;
	/* nb answers */
	case 7:
		answers=(w16<<8)|c;
		sprintf(s,"/%u",answers);
		DOMAINloggue(s);
		break;
	/* nb authority */
	case 9:
		authority=(w16<<8)|c;
		sprintf(s,"/%u",authority);
		DOMAINloggue(s);
		break;
	/* nb additionnal */
	case 11:
		additionnal=(w16<<8)|c;
		if (additionnal) {
			sprintf(s,"/%u",additionnal);
			DOMAINloggue(s);
		}
		break;
	case 12:
		DOMAINloggue(")");
		state=BEGIN;
		current=0;
	default:
		if (state==BEGIN) {
			if (current==-1) return; /* garbage at end */
			if (current) DOMAINloggue(";");
			while (!current) {
				DOMAINloggue("|");
				current=questions;
				questions=answers;
				answers=authority;
				authority=additionnal;
				additionnal=-1;
			}
			current--;
			state=NAME;
			namelen=0;
		}
		switch(state) {
		case NAME:
			if (namelen==0) {
				if (!c) {
					state=TYPEANDCO;
					domainpointeur=0;
				}
				else if (c>63) {
					DOMAINloggue("@");
					namelen=-1;
				} else namelen=c;
			} else if (namelen == -1) {
				state=TYPEANDCO;
				domainpointeur=0;
			} else {
				sprintf(s,"%c",c);
				DOMAINloggue(s);
				if (!(--namelen)) DOMAINloggue(".");
			}
			break;
		case TYPEANDCO:
			switch(domainpointeur) {
			case 0:
			case 2:
			case 8:
				w16=c;
				break;
			/* TYPE */
			case 1:
				type=(w16<<8) | c;
				switch(type) {
				case 1: DOMAINloggue(" A"); break;
				case 2: DOMAINloggue(" NS"); break;
				case 3: DOMAINloggue(" MD"); break;
				case 4: DOMAINloggue(" MF"); break;
				case 5: DOMAINloggue(" CNAME"); break;
				case 6: DOMAINloggue(" SOA"); break;
				case 7: DOMAINloggue(" MB"); break;
				case 8: DOMAINloggue(" MG"); break;
				case 9: DOMAINloggue(" MR"); break;
				case 10: DOMAINloggue(" NULL"); break;
				case 11: DOMAINloggue(" WKS"); break;
				case 12: DOMAINloggue(" PTR"); break;
				case 13: DOMAINloggue(" HINFO"); break;
				case 14: DOMAINloggue(" MINFO"); break;
				case 15: DOMAINloggue(" MX"); break;
				case 16: DOMAINloggue(" TXT"); break;
				default: DOMAINloggue(" unkn"); break;
				}
				break;
			/* class */
			case 3:
				switch((w16<<8)|c) {
				case 1: DOMAINloggue(" IN"); break;
				case 2: DOMAINloggue(" CS"); break;
				case 3: DOMAINloggue(" CH"); break;
				case 4: DOMAINloggue(" HS"); break;
				default: DOMAINloggue(" unkn"); break;
				}
				if (authority!=-1) state=BEGIN;
				break;
			/* TTL */
			case 4: w32=c; break;
			case 5: case 6: 
				w32=(w32<<8L) | c;
				break;
			case 7:
				sprintf(s," (%luj)",((w32<<8L)|c)/(24L*3600L));
				DOMAINloggue(s);
				break;
			/* RDLENGTH */
			case 9:
				size=(w16<<8) | c;
				DOMAINloggue(":");
				break;
			default:
				if ((domainpointeur-10)==size-1) state=BEGIN;
				switch (type) {
				case 1: /* A */
					if (domainpointeur-10<3)
						sprintf(s,"%u.",c);
					else
						sprintf(s,"%u",c);
					DOMAINloggue(s);
					break;
				case 2: /* NS */
				case 3: /* MD */
				case 4: /* MF */
				case 5: /* CNAME */
				case 7: /* MB */
				case 8: /* MG */
				case 9: /* MR */
				case 12: /* PTR */
					if (domainpointeur-10==0) namelen=0;
					if (namelen==0) {
						if (c) {
						  if (c>63) {
							DOMAINloggue("@");
							namelen = -1;
						  } else namelen=c;
						}
					} else if (namelen == -1) {
						namelen = 0;
					} else {
						sprintf(s,"%c",c);
						DOMAINloggue(s);
						if (!(--namelen)) DOMAINloggue(".");
					}
					break;
				case 13: /* HINFO */
				case 16: /* TXT */
					sprintf(s,"%c",c);
					DOMAINloggue(s);
					break;
				default: if (domainpointeur-10==0) {
						 sprintf(s,"(%u)",size);
						 DOMAINloggue(s);
					 }
					 break;
				}
				break;
			}
			domainpointeur++;
			break;
		}
		break;
	}
	applipointeur++;
}
