/*
 *  Copyright (C) Samuel Thibault <samuel.thibault@fnac.net>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 * ip.c
 *
 * a ppp analyser for Unix
 *
 * (c) Copyright 2000-2001 Samuel Thibault
 */

#include <stdio.h>
#include "analyse.h"
#include "codes.h"

#define IPloggue(s)	if (show.ip) loggue(s);

short ippointeur;
short protopointeur;
short applipointeur;

extern short state;

struct protos_s ipprotos[] = {
	{ 0,	"reserved" },
	{ 1,	"ICMP" },
	{ 2,	"IGMP" },
	{ 3,	"GGP" },
	{ 4,	"IP" },
	{ 5,	"ST" },
	{ 6,	"TCP" },
	{ 7,	"UCL" },
	{ 8,	"EGP" },
	{ 9,	"IGP" },
	{ 10,	"BBN-RCC-MON" },
	{ 11,	"NVP-II" },
	{ 12,	"PUP" },
	{ 13,	"ARGUS" },
	{ 14,	"EMCON" },
	{ 15,	"XNET" },
	{ 16,	"CHAOS" },
	{ 17,	"UDP" },
	{ 18,	"MUX" },
	{ 19,	"DCN-MEAS" },
	{ 20,	"HMP" },
	{ 21,	"PRM" },
	{ 22,	"XNS-IDP" },
	{ 23,	"TRUNK-1" },
	{ 24,	"TRUNK-2" },
	{ 25,	"LEAF-1" },
	{ 26,	"LEAF-2" },
	{ 27,	"RDP" },
	{ 28,	"IRTP" },
	{ 29,	"ISO-TP4" },
	{ 30,	"NETBLT" },
	{ 31,	"MFE-NSP" },
	{ 32,	"MERIT-INP" },
	{ 33,	"SEP" },
	{ 34,	"3PC" },
	{ 35,	"IDPR" },
	{ 36,	"XTP" },
	{ 37,	"DDP" },
	{ 38,	"IDPR-CMTP" },
	{ 39,	"TP++" },
	{ 40,	"IL" },
	{ 61,	"internal" },
	{ 62,	"CFTP" },
	{ 63,	"local network" },
	{ 64,	"SAT-EXPAK" },
	{ 65,	"KRYPTOLAN" },
	{ 66,	"RVD" },
	{ 67,	"IPPC" },
	{ 68,	"distributed file system" },
	{ 69,	"SAT-MON" },
	{ 70,	"VISA" },
	{ 71,	"IPCV" },
	{ 72,	"CPNX" },
	{ 73,	"CPHB" },
	{ 74,	"WSN" },
	{ 75,	"PVP" },
	{ 76,	"BR-SAT-MON" },
	{ 77,	"SUN-ND" },
	{ 78,	"WB-MON" },
	{ 79,	"WB-EXPAK" },
	{ 80,	"ISO-IP" },
	{ 81,	"VMTP" },
	{ 82,	"SECURE-VMTP" },
	{ 83,	"VINES" },
	{ 84,	"TTP" },
	{ 85,	"NSFNET-IGP" },
	{ 86,	"DGP" },
	{ 87,	"TCF" },
	{ 88,	"IGRP" },
	{ 89,	"OSPFIGP" },
	{ 90,	"Sprite-RPC" },
	{ 91,	"LARP" },
	{ 92,	"MTP" },
	{ 93,	"AX.25" },
	{ 94,	"IPIP" },
	{ 95,	"MICP" },
	{ 96,	"AES-SP3-D" },
	{ 97,	"ETHERIP" },
	{ 98,	"ENCAP" },
	{ 255,	"reserved" },
	{ -1,	NULL }
};

void analyseip(unsigned char c) {
	static char s[64];
	static short lgent;
	static unsigned int size;
	static unsigned char proto;

	switch (ippointeur) {
/* ipv | lgent */
	case 0:
		protopointeur=0;
		if ((c & 0xf0) != 0x40) {
			sprintf(s,"v%d",(c & 0xf0) >> 4);
			IPloggue(s);
			state=discard;
		} else {
			lgent=c & 0x0f;
		}
		break;
/* TOS */
	case 1:
		break;
/* Size */
	case 2:
		size=c;
		break;
	case 3:
		size=(size << 8)+c;
		break;
/* Id */
	case 4:
	case 5:
/* Options */
	case 6:
	case 7:
		break;
/* ttl */
	case 8:
		sprintf(s, " t%d", c);
		IPloggue(s);
		break;
/* protocol */
	case 9:
		proto=c;
		IPloggue(" ");
		IPloggue(findproto(ipprotos,c));
		break;
/* chksum */
	case 10:
	case 11:
		break;
/* ip src */
	case 16:
		IPloggue(" >");
	case 12:
		IPloggue(" ");
	case 13:
	case 14:
	case 17:
	case 18:
		sprintf(s,"%d.",c);
		IPloggue(s);
		break;
	case 15:
	case 19:
		sprintf(s,"%d",c);
		IPloggue(s);
		break;
	default:
		if (ippointeur>=lgent<<2 && ippointeur < size) switch (proto) {
		case ICMP:
			analyseicmp(c);
			break;
		case TCP:
			analysetcp(c);
			break;
		case UDP:
			analyseudp(c);
			break;
		}
	}
	ippointeur++;
}
