/*
 *  Copyright (C) Samuel Thibault <samuel.thibault@fnac.net>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 * ppp.c
 *
 * a ppp analyser for Unix
 *
 * (c) Copyright 2000-2001 Samuel Thibault
 */

#include <stdio.h>
#include "analyse.h"
#include "codes.h"

u16 fcs;
short tossing=1;
short tossed=0;
int escaped;
short state;
short proto;
unsigned int asyncmap=0xFFFF;		/* all are escaped by default ! */

extern unsigned char *source;
extern unsigned char *desti;

extern unsigned char *pos;
extern unsigned char *current;

extern short statecp;
extern short stateauth;
extern short curcp;

extern short ippointeur;

struct protos_s unsupported_0x00[] = {
	{ 0x01,	"PadP" },
	{ 0x23,	"OSI" },
	{ 0x25,	"XEROX" },
	{ 0x27,	"DECnet" },
	{ 0x29,	"Appletalk" },
	{ 0x2B,	"IPX" },
	{ 0x2D,	"VJ comp" },
	{ 0x2F,	"VJ uncomp" },
	{ 0x31,	"Bridging PDU" },
	{ 0x33,	"Stream Protocol (ST-II)" },
	{ 0x35,	"Banyan Vines" },
	{ 0x39,	"Appletalk EDDP" },
	{ 0x3b,	"AppleTalk SmartBuffered" },
	{ 0x3d,	"Multi-Link" },
	{ 0x3f,	"NETBIOS Framing" },
	{ 0x41,	"Cisco Systems" },
	{ 0x43,	"Ascom Timeplex" },
	{ 0x45,	"Fujitsu Link Backup and Load Balancing (LBLB)" },
	{ 0x47,	"DCA Remote Lan" },
	{ 0x49,	"Serial Data Transport Protocol (PPP-SDTP)" },
	{ 0x4b,	"SNA over 802.2" },
	{ 0x4d,	"SNA" },
	{ 0x4f,	"IP6 Header Compression" },
	{ 0x6f,	"Stampede Bridging" },
	{ 0xfb,	"single-link compression" },
	{ 0xfd,	"1st choice compression" },
	{ 0,	NULL }
};

struct protos_s unsupported_0x02[] = {
	{ 0x01,	"802.1d Hello Packet" },
	{ 0x03,	"IBM Source Routing BPDU" },
	{ 0x05,	"DEC LANBridge100 Spanning Tree" },
	{ 0x31,	"Luxcom" },
	{ 0x33,	"Sigma Network Systems" },
	{ 0,	NULL }
};

struct protos_s unsupported_0x80[] = {
	{ 0x23,	"OSI Network Control Protocol" },
	{ 0x25,	"XEROX NS IDP Control Protocol" },
	{ 0x27,	"DECnet Phase IV Control Protocol" },
	{ 0x29,	"Appletalk Control Protocol" },
	{ 0x2B,	"Novell IPX Control Protocol" },
	{ 0x31,	"Bridging NCP" },
	{ 0x33,	"Stream Protocol Control Protocol" },
	{ 0x35,	"Banyan Vines Control Protocol" },
	{ 0x3D,	"Multi-Link Control Protocol" },
	{ 0x3f,	"NETBIOS Framing Control Protocol" },
	{ 0x41,	"Cisco Systems Control Protocol" },
	{ 0x43,	"Ascom Timeplex" },
	{ 0x45,	"Fujitsu LBLB Control Protocol" },
	{ 0x47,	"DCA Remote Lan Network Control Protocol (RLNCP)" },
	{ 0x49,	"Serial Data Control Protocol (PPP-SDCP)" },
	{ 0x4b,	"SNA over 802.2 Control Protocol" },
	{ 0x4d,	"SNA Control Protocol" },
	{ 0x4f,	"IP6 Header Compression Control Protocol" },
	{ 0x6f,	"Stampede Bridging Control Protocol" },
	{ 0xfb,	"Single Link Compression Control Protocol" },
	{ 0xFD,	"Compression Control Protocol" },
	{ 0,	NULL }
};

struct protos_s unsupported_0xC0[] = {
	{ 0x25,	"Link Quality Report" },
	{ 0x27,	"Shiva Password Authentication Protocol" },
	{ 0x29,	"CallBack Control Protocol (CBCP)" },
	{ 0x81,	"Container Control Protocol" },
	{ 0,	NULL }
};

struct protos_s unsupported_0xC2[] = {
	{ 0x81,	"Proprietary Authentication Protocol" },
	{ 0,	NULL }
};

void network_protos(unsigned char c) {
	switch (c) {
	case 0x21:
		if (show.ip)
			loggue(" IP");
		state=SPPP_IP;
		ippointeur=0;
		break;
	default:
		if (show.others) {
			loggue(" ");
			loggue(findproto(unsupported_0x00,c));
		}
		state=discard;
	}
}

/*
 * analyses a full cleaned frame : checks address & control, frame type
 */

void analyseppp(unsigned char c) {
	switch (state) {
	case SPPP_ADDRESS:
		if (c==0xFF) state=SPPP_CONTROL;
		else {
			state=SPPP_PROTOCOL;
			analyseppp(c);
		}
		break;
	case SPPP_CONTROL:
		if (c==0x03) state=SPPP_PROTOCOL;
		else {
			loggue(" EControl");
			state=SPPP_PROTOCOL;
			analyseppp(c);
		}
		break;
	case SPPP_PROTOCOL:
		switch (proto) {	// initially 0xFF (see trame_begin)
		case 0xFF:
			proto=c;	// remember for next turn
			break;
		case 0x02:
			if (show.others) {
				loggue(" ");
				loggue(findproto(unsupported_0x02,c));
			}
			state=discard;
			break;
		case 0x80:
			switch (c) {
			case 0x21:
				if (show.cp)
					loggue(" IPCP");
				state=SPPP_CP;
				statecp=SCP_CODE;
				curcp=IPCP;
				break;
			default:
				if (show.others) {
					loggue(" ");
					loggue(findproto(unsupported_0x80,c));
				}
				state=discard;
				break;
			}
			break;
		case 0xC0:
			switch (c) {
			case 0x21:
				if (show.cp)
					loggue(" LCP");
				state=SPPP_CP;
				statecp=SCP_CODE;
				curcp=LCP;
				break;
			case 0x23:
				if (show.cp)
					loggue(" PAP");
				state=SPPP_PAP;
				stateauth=SAUTH_CODE;
				break;
			default:
				if (show.others) {
					loggue(" ");
					loggue(findproto(unsupported_0xC0,c));
				}
				state=discard;
				break;
			}
			break;
		case 0xC2:
			if (c==0x23) {
				if (show.cp)
					loggue(" CHAP");
				state=SPPP_CHAP;
				stateauth=SAUTH_CODE;
			} else {
				if (show.others) {
					loggue(" ");
					loggue(findproto(unsupported_0xC0,c));
				}
				state=discard;
			}
			break;
		case 0x00:
			network_protos(c);
			break;
		default:
			network_protos(proto);
			analyseppp(c);
			break;
		}
		break;
	case SPPP_CP:
		analysecp(c);
		break;
	case SPPP_IP:
		analyseip(c);
		break;
	case SPPP_PAP:
		analysepap(c);
		break;
	case SPPP_CHAP:
		analysechap(c);
		break;
	case discard:
		break;
	}
}

/*
 * just to say a char was received (analyseppp can be called from itself !)
 */

void analysechar(unsigned char c) {
	fcs=pppfcs(fcs,c);
	analyseppp(c);
}

/*
 * manages frames, divided by flags. Also manages escaped chars
 */

void analyse(unsigned char c) {
	if (tossing) {
		if (c==PPP_FLAG) {
			tossing=0;
			if (tossed) fprintf(stderr,"\r\n");
			trame_begin();
		} else {
			tossed=1;
			fprintf(stderr,"%c",c);
		}
	} else if ((show.xonxoff) && ((c==START_CHAR) || (c==STOP_CHAR))) {
		fprintf(stderr,c==START_CHAR ? "q\r\n" : "s\r\n");
	} else if (escaped)
		if (c==PPP_FLAG) {
			loggue("frame error\r\n");
			flush_log();
			trame_begin();
		} else {
			c^=0x20;
			if (c < ' ' && ! (asyncmap & (1 << c))) {
				if (show.asyncmap)
					fprintf(stderr, "%s: 0x%x was escaped !\r\n", source, c);
				asyncmap |= 1 << c;
			}
			analysechar(c);
			escaped=0;
		}
	else
		switch (c) {
		case PPP_ESCAPE:
			escaped=1;
			break;
		case PPP_FLAG:
			if (state!=SPPP_ADDRESS) {
				loggue((fcs == PPPGOODFCS) ? "\r\n" : " bad FCS\r\n");
				flush_log();
			}
			trame_begin();
			break;
		default:
			if (c < ' ' && asyncmap & (1 << c)) {
				asyncmap &= ~(1 << c);
				if (show.asyncmap)
					fprintf(stderr, "%s: 0x%x wasn't escaped !\r\n", source, c);
			}
			if (c >= ' ' || ! (asyncmap & (1 << c)))
				analysechar(c);
			break;
		}	
}

void trame_begin(void) {
	fcs=PPPINITFCS;
	pos=current;
	loggue(source);
//	loggue("->");
//	loggue(desti);
	escaped=0;
	state=SPPP_ADDRESS;
	proto=0xFF;
}
