/*
 *  Copyright (C) Samuel Thibault <samuel.thibault@fnac.net>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 * utils.c
 *
 * a ppp analyser for Unix
 *
 * (c) Copyright 2000-2001 Samuel Thibault
 */

#include <stdio.h>
#include "analyse.h"

char *findproto(struct protos_s *protos, short unsigned code) {
	static char s[14];
	struct protos_s *cur;
	for (cur=protos;(cur->info) && (cur->code != code); cur++);
	if (!cur->info) {
		sprintf(s,"unknown: 0x%x",code);
		return(s);
	}
	return cur->info;
}

struct protos_s services[] = {
	{ 1,	"tcpmux" },
	{ 7,	"echo" },
	{ 7,	"echo" },
	{ 9,	"discard" },
	{ 9,	"discard" },
	{ 11,	"systat" },
	{ 13,	"daytime" },
	{ 13,	"daytime" },
	{ 15,	"netstat" },
	{ 17,	"qotd" },
	{ 18,	"msp" },
	{ 18,	"msp" },
	{ 19,	"chargen" },
	{ 19,	"chargen" },
	{ 20,	"ftp-data" },
	{ 21,	"ftp" },
	{ 21,	"fsp" },
	{ 22,	"ssh" },
	{ 22,	"ssh" },
	{ 23,	"telnet" },
	{ 24,	"private" },
	{ 25,	"smtp" },
	{ 26,	"unassigned" },
	{ 37,	"time" },
	{ 37,	"time" },
	{ 39,	"rlp" },
	{ 42,	"nameserver" },
	{ 43,	"whois" },
	{ 50,	"re-mail-ck" },
	{ 50,	"re-mail-ck" },
	{ 53,	"domain" },
	{ 53,	"domain" },
	{ 57,	"mtp" },
	{ 67,	"bootps" },
	{ 67,	"bootps" },
	{ 68,	"bootpc" },
	{ 68,	"bootpc" },
	{ 69,	"tftp" },
	{ 70,	"gopher" },
	{ 70,	"gopher" },
	{ 77,	"rje" },
	{ 79,	"finger" },
	{ 80,	"www" },
	{ 80,	"www" },
	{ 87,	"link" },
	{ 88,	"kerberos" },
	{ 88,	"kerberos" },
	{ 95,	"supdup" },
	{ 100,	"reserved" },
	{ 101,	"hostnames" },
	{ 102,	"iso-tsap" },
	{ 105,	"csnet-ns" },
	{ 105,	"csnet-ns" },
	{ 106,	"3com-tsmux" },
	{ 106,	"3com-tsmux" },
	{ 107,	"rtelnet" },
	{ 107,	"rtelnet" },
	{ 109,	"pop-2" },
	{ 109,	"pop-2" },
	{ 110,	"pop-3" },
	{ 110,	"pop-3" },
	{ 111,	"sunrpc" },
	{ 111,	"sunrpc" },
	{ 113,	"auth" },
	{ 115,	"sftp" },
	{ 117,	"uucp-path" },
	{ 119,	"nntp" },
	{ 123,	"ntp" },
	{ 123,	"ntp" },
	{ 137,	"netbios-ns" },
	{ 137,	"netbios-ns" },
	{ 138,	"netbios-dgm" },
	{ 138,	"netbios-dgm" },
	{ 139,	"netbios-ssn" },
	{ 139,	"netbios-ssn" },
	{ 143,	"imap2" },
	{ 143,	"imap2" },
	{ 161,	"snmp" },
	{ 162,	"snmp-trap" },
	{ 163,	"cmip-man" },
	{ 163,	"cmip-man" },
	{ 164,	"cmip-agent" },
	{ 164,	"cmip-agent" },
	{ 177,	"xdmcp" },
	{ 177,	"xdmcp" },
	{ 178,	"nextstep" },
	{ 178,	"nextstep" },
	{ 179,	"bgp" },
	{ 179,	"bgp" },
	{ 191,	"prospero" },
	{ 191,	"prospero" },
	{ 194,	"irc" },
	{ 194,	"irc" },
	{ 199,	"smux" },
	{ 199,	"smux" },
	{ 201,	"at-rtmp" },
	{ 201,	"at-rtmp" },
	{ 202,	"at-nbp" },
	{ 202,	"at-nbp" },
	{ 204,	"at-echo" },
	{ 204,	"at-echo" },
	{ 206,	"at-zis" },
	{ 206,	"at-zis" },
	{ 209,	"qmtp" },
	{ 209,	"qmtp" },
	{ 210,	"z3950" },
	{ 210,	"z3950" },
	{ 213,	"ipx" },
	{ 213,	"ipx" },
	{ 220,	"imap3" },
	{ 220,	"imap3" },
	{ 369,	"rpc2portmap" },
	{ 369,	"rpc2portmap" },
	{ 370,	"codaauth2" },
	{ 370,	"codaauth2" },
	{ 372,	"ulistserv" },
	{ 372,	"ulistserv" },
	{ 443,	"https" },
	{ 443,	"https" },
	{ 444,	"snpp" },
	{ 444,	"snpp" },
	{ 487,	"saft" },
	{ 487,	"saft" },
	{ 610,	"npmp-local" },
	{ 610,	"npmp-local" },
	{ 611,	"npmp-gui" },
	{ 611,	"npmp-gui" },
	{ 612,	"hmmp-ind" },
	{ 612,	"hmmp-ind" },
	{ 512,	"exec" },
	{ 512,	"biff" },
	{ 513,	"login" },
	{ 513,	"who" },
	{ 514,	"shell" },
	{ 514,	"syslog" },
	{ 515,	"printer" },
	{ 517,	"talk" },
	{ 518,	"ntalk" },
	{ 520,	"route" },
	{ 525,	"timed" },
	{ 526,	"tempo" },
	{ 530,	"courier" },
	{ 531,	"conference" },
	{ 532,	"netnews" },
	{ 533,	"netwall" },
	{ 540,	"uucp" },
	{ 548,	"afpovertcp" },
	{ 548,	"afpovertcp" },
	{ 556,	"remotefs" },
	{ 543,	"klogin" },
	{ 544,	"kshell" },
	{ 749,	"kerberos-adm" },
	{ 765,	"webster" },
	{ 765,	"webster" },
	{ 1524,	"ingreslock" },
	{ 1524,	"ingreslock" },
	{ 1525,	"prospero-np" },
	{ 1525,	"prospero-np" },
	{ 1645,	"datametrics" },
	{ 1645,	"datametrics" },
	{ 1646,	"sa-msg-port" },
	{ 1646,	"sa-msg-port" },
	{ 1812,	"radius" },
	{ 1812,	"radius" },
	{ 1813,	"radacct" },
	{ 1813,	"radacct" },
	{ 2401,	"cvspserver" },
	{ 2401,	"cvspserver" },
	{ 2430,	"venus" },
	{ 2430,	"venus" },
	{ 2431,	"venus-se" },
	{ 2431,	"venus-se" },
	{ 2432,	"codasrv" },
	{ 2432,	"codasrv" },
	{ 2433,	"codasrv-se" },
	{ 2433,	"codasrv-se" },
	{ 3306,	"mysql" },
	{ 3306,	"mysql" },
	{ 5002,	"rfe" },
	{ 5002,	"rfe" },
	{ 5308,	"cfengine" },
	{ 5308,	"cfengine" },
	{ 7000,	"bbs" },
	{ 750,	"kerberos4" },
	{ 750,	"kerberos4" },
	{ 751,	"kerberos_master" },
	{ 751,	"kerberos_master" },
	{ 752,	"passwd_server" },
	{ 754,	"krb_prop" },
	{ 760,	"krbupdate" },
	{ 761,	"kpasswd" },
	{ 1109,	"kpop" },
	{ 2053,	"knetd" },
	{ 2102,	"zephyr-srv" },
	{ 2103,	"zephyr-clt" },
	{ 2104,	"zephyr-hm" },
	{ 2105,	"eklogin" },
	{ 871,	"supfilesrv" },
	{ 1127,	"supfiledbg" },
	{ 1,	"rtmp" },
	{ 2,	"nbp" },
	{ 4,	"echo" },
	{ 6,	"zip" },
	{ 106,	"poppassd" },
	{ 106,	"poppassd" },
	{ 174,	"mailq" },
	{ 174,	"mailq" },
	{ 465,	"ssmtp" },
	{ 538,	"gdomap" },
	{ 538,	"gdomap" },
	{ 563,	"snews" },
	{ 636,	"ssl-ldap" },
	{ 808,	"omirr" },
	{ 808,	"omirr" },
	{ 873,	"rsync" },
	{ 873,	"rsync" },
	{ 993,	"simap" },
	{ 995,	"spop3" },
	{ 1080,	"socks" },
	{ 1080,	"socks" },
	{ 1236,	"rmtcfg" },
	{ 1313,	"xtel" },
	{ 1529,	"support" },
	{ 2003,	"cfinger" },
	{ 2150,	"ninstall" },
	{ 2150,	"ninstall" },
	{ 2988,	"afbackup" },
	{ 2988,	"afbackup" },
	{ 3130,	"icp" },
	{ 3130,	"icp" },
	{ 5432,	"postgres" },
	{ 5432,	"postgres" },
	{ 4557,	"fax" },
	{ 4559,	"hylafax" },
	{ 5354,	"noclog" },
	{ 5354,	"noclog" },
	{ 5355,	"hostmon" },
	{ 5355,	"hostmon" },
	{ 5900,	"vnc0" },
	{ 5901,	"vnc1" },
	{ 5902,	"vnc2" },
	{ 5903,	"vnc3" },
	{ 5904,	"vnc4" },
	{ 6000,	"X0" },
	{ 6001,	"X1" },
	{ 6002,	"X2" },
	{ 6003,	"X3" },
	{ 6004,	"X4" },
	{ 6667,	"ircd" },
	{ 6667,	"ircd" },
	{ 8080,	"webcache" },
	{ 8080,	"webcache" },
	{ 8081,	"tproxy" },
	{ 8081,	"tproxy" },
	{ 9359,	"mandelspawn" },
	{ 10080,	"amanda" },
	{ 10081,	"kamanda" },
	{ 10081,	"kamanda" },
	{ 10082,	"amandaidx" },
	{ 10083,	"amidxtape" },
	{ 20011,	"isdnlog" },
	{ 20011,	"isdnlog" },
	{ 20012,	"vboxd" },
	{ 20012,	"vboxd" },
	{ 24554,	"binkp" },
	{ 24554,	"binkp" },
	{ 27374,	"asp" },
	{ 27374,	"asp" },
	{ 60177,	"tfido" },
	{ 60177,	"tfido" },
	{ 60179,	"fido" },
	{ 60179,	"fido" },
	{ 98,	"linuxconf" },
	{ 901,	"swat" },
	{ 31457,	"tetrinet" },
	{ 2064,	"dnetc" },
	{ -1,	NULL }
};

char *findservice(unsigned int code) {
	static char s[14];
	struct protos_s *cur;
	for (cur=services;(cur->info) && (cur->code != code); cur++);
	if (!cur->info) {
		sprintf(s,"%u",code);
		return(s);
	}
	return cur->info;
}
